"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const command_1 = require("@oclif/command");
const fs_1 = __importDefault(require("fs"));
const osu_json_parser_1 = __importDefault(require("osu-json-parser"));
const path_1 = __importDefault(require("path"));
const readline_1 = __importDefault(require("readline"));
const parseRGB_1 = __importDefault(require("./parseRGB"));
async function start(file, name, musicExt, offset) {
    let filepath = `${path_1.default.basename(name)}.inc`;
    filepath = path_1.default.join(path_1.default.dirname(file), filepath);
    const rl = readline_1.default.createInterface({
        input: fs_1.default.createReadStream(file),
    });
    const parser = new osu_json_parser_1.default();
    return new Promise((resolve) => {
        rl.on("line", (line) => {
            parser.parse(line);
        });
        rl.on("close", () => {
            const scripts = parseRGB_1.default(name, parser.toJSON(), musicExt.replace(".", ""), offset);
            fs_1.default.writeFileSync(filepath, scripts);
            resolve();
        });
    });
}
class RGBOsu extends command_1.Command {
    async run() {
        const { args, flags } = this.parse(RGBOsu);
        if (args.file) {
            const offset = Number(flags.offset);
            const name = flags.name.replace(/[^\x00-\x7F]/g, "");
            start(args.file, path_1.default.basename(name, path_1.default.extname(name)), path_1.default.extname(name), Number.isNaN(offset) ? 0 : offset);
        }
    }
}
RGBOsu.description = "OSU! 채보를 RGB 채보로 변환합니다.";
RGBOsu.flags = {
    name: command_1.flags.string({ char: "n", required: true, description: "음원 이름 (example.mp3)" }),
    offset: command_1.flags.string({ char: "o", description: "offset (milliseconds)" }),
};
RGBOsu.args = [{ name: "file" }];
module.exports = RGBOsu;
