"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const rgbDelay = 1.5781;
function parseRGB(name, osu, musicExt, offset) {
    const musicScripts = parseOsu(osu, offset);
    return makePawnFunction(name, musicScripts, musicExt);
}
function makePawnFunction(name, musicScripts, musicExt) {
    const scripts = [];
    scripts.push("#include \"rgb/base.inc\"");
    scripts.push("");
    scripts.push(`public void Start_${name}()`);
    scripts.push("{");
    scripts.push(`  StartMusic(${rgbDelay}, "${name}.${musicExt}");`);
    scripts.push(musicScripts.map((str) => `  ${str}`).join("\n"));
    scripts.push("}");
    return scripts.join("\n");
}
function parseOsu(osu, offset = 0) {
    const { difficulty: { SliderMultiplier }, timingPoints: _timingPoints, hitObjects } = osu;
    const scripts = hitObjects.flatMap((hitobject) => {
        const { StartTime, RepeatCount, Samples, NodeSamples } = hitobject;
        let timingPoints = _timingPoints
            .filter(({ Time }) => Time < StartTime)
            .sort(({ Time }) => Time)
            .reverse();
        if (timingPoints.length === 0) {
            timingPoints = _timingPoints.sort(({ Time }) => Time);
        }
        const currentTimingPoint = timingPoints[0];
        const time = (StartTime + offset) / 1000;
        if (RepeatCount !== undefined && RepeatCount >= 0) {
            const [{ BeatLength: beatLength }] = timingPoints
                .filter(({ BeatLength }) => BeatLength);
            const [{ SpeedMultiplier: speedMultiplier }] = timingPoints;
            const { Length = 0 } = hitobject;
            const sliderDuration = Length / (100.0 * SliderMultiplier) * (beatLength * speedMultiplier) / 1000;
            const scripts = [];
            for (let i = 0; i < RepeatCount + 2; i++) {
                scripts.push(CreateNoteScript(time + (sliderDuration * i), currentTimingPoint, NodeSamples[i]));
            }
            return scripts;
        }
        return CreateNoteScript(time, currentTimingPoint, Samples);
    });
    return scripts;
}
function CreateNoteScript(time, timingPoint, samples) {
    const { SampleBank: sampleBank, SampleVolume: volume } = timingPoint;
    const flags = {
        hitnormal: "NORMAL",
        hitwhistle: "WHISTLE",
        hitfinish: "FINISH",
        hitclap: "CLAP",
    };
    const banks = {
        normal: "NORMAL",
        soft: "SOFT",
        drum: "DRUM",
    };
    const noteSamples = samples.map(({ Bank: bank, Name: name = "hitnormal" }) => {
        return `NOTE_${banks[bank || sampleBank]}_${flags[name]}`;
    });
    let soundVolume;
    if (samples[0].Volume) {
        soundVolume = samples[0].Volume;
    }
    else {
        soundVolume = volume;
    }
    soundVolume = (soundVolume / 100).toFixed(1);
    return `Note(${time.toFixed(4)}, ${noteSamples.join("|")}, ${soundVolume});`;
}
exports.default = parseRGB;
